package com.Ekyc.ekyc2022

import android.Manifest
import android.annotation.SuppressLint
import android.content.Context
import android.content.Intent
import android.content.pm.PackageManager
import android.content.res.ColorStateList
import android.graphics.Bitmap
import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.view.WindowManager
import android.widget.Toast
import androidx.activity.result.contract.ActivityResultContract
import androidx.appcompat.app.AppCompatActivity
import androidx.constraintlayout.widget.ConstraintLayout
import androidx.constraintlayout.widget.ConstraintSet
import androidx.core.app.ActivityCompat
import androidx.core.content.ContextCompat
import com.Ekyc.ekyc2022.databinding.ActivityPreviewBinding
import com.Ekyc.ekyc2022.camerax.CameraManager
import com.Ekyc.ekyc2022.camerax.CameraMode
import com.Ekyc.ekyc2022.camerax.CameraOption
import com.Ekyc.ekyc2022.databinding.ActivityPreviewFaceBinding
import com.Ekyc.ekyc2022.facedetection.AnalyzeResult
import com.Ekyc.ekyc2022.facedetection.FaceContourDetectionProcessor
import com.Ekyc.ekyc2022.global.Constants
import com.Ekyc.ekyc2022.global.Utils.writeBitmapToFile
import com.Ekyc.ekyc2022.objectdetection.AlignCardResult
import com.Ekyc.ekyc2022.objectdetection.AlignIdImageHelper
import com.Ekyc.ekyc2022.objectdetection.CardCornersDetectionProcessor
import com.Ekyc.ekyc2022.objectdetection.IdCardType
import java.util.*

class CameraPreviewActivity : AppCompatActivity() {
    private lateinit var cameraManager: CameraManager
    private var faceDetectionProcessor: FaceContourDetectionProcessor? = null
    private var cornersDetectionProcessor: CardCornersDetectionProcessor? = null
    private var analyzeTimer: Timer? = null
    private var analyzeTimerTask: TimerTask? = null
    private var hasStartedCamera = false
    private lateinit var overlayTintList: ColorStateList
    private var isCaptured = false
    private var enableAutoCapture = Application.enableAuto
    private var mode = MODE_TAKE_SELFIE
    private var verifyProfileStep = VerifyProfileStep.FRONT_ID_CARD
    private var borderMarginHorizontal = 0
    private var borderMarginVertical = 0
    private lateinit var binding: ActivityPreviewBinding
    private var count: Int = 0

    // test

    companion object {
        const val MODE_TAKE_SELFIE = 169
        const val MODE_CAPTURE_FRONT_ID_CARD = 269
        const val MODE_CAPTURE_BACK_ID_CARD = 369

        private const val REQUEST_CODE_PERMISSIONS = 6969
        private val REQUIRED_PERMISSIONS =
            arrayOf(Manifest.permission.CAMERA)
        private const val ANALYZE_PERIOD = 200L
    }

    private fun updateStatusBarColor() {
        val window = this.window
        window.addFlags(WindowManager.LayoutParams.FLAG_DRAWS_SYSTEM_BAR_BACKGROUNDS)
        window.clearFlags(WindowManager.LayoutParams.FLAG_TRANSLUCENT_STATUS);
        window.statusBarColor = ContextCompat.getColor(this, R.color.black)
        window.navigationBarColor = ContextCompat.getColor(this, R.color.black) //
    }


    override fun onNewIntent(intent: Intent?) {
        super.onNewIntent(intent)
        getIntent(intent)
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        getIntent(intent)
        val view = if (mode == MODE_TAKE_SELFIE)
            ActivityPreviewFaceBinding.inflate(layoutInflater).root else
            ActivityPreviewBinding.inflate(layoutInflater).root
        binding = ActivityPreviewBinding.bind(view)
        setContentView(binding.root)
        binding.ivTakePhoto.setOnClickListener {
            if (enableAutoCapture) {
                return@setOnClickListener
            }
            if (!isCaptured) {
                isCaptured = true
                cornersDetectionProcessor?.process(binding.previewView.bitmap, true)
            }
        }
        binding.tvCancel.setOnClickListener {
            val intent = Intent()
            intent.putExtra(Constants.KEY_CANCEL, true)
            setResult(RESULT_OK, intent)
            finish()
        }
        setupViews()
        checkPermissions()
        updateStatusBarColor()
    }

    override fun onBackPressed() {
        val intent = Intent()
        intent.putExtra(Constants.KEY_CANCEL, true)
        setResult(RESULT_OK, intent)
        super.onBackPressed()
    }

    private fun setupViews() {
        val title: String = when (mode) {
            MODE_TAKE_SELFIE -> {
                "Xác thực khuôn mặt"
            }
            MODE_CAPTURE_FRONT_ID_CARD -> {
                "Mặt trước"
            }
            else -> {
                "Mặt sau"
            }
        }
        binding.textViewTitle.text = title
        val set = ConstraintSet()
        set.clone(binding.root)
        set.setDimensionRatio(
            R.id.previewView,
            if (mode == MODE_TAKE_SELFIE) "1:1" else "4:3"
        )
        set.applyTo(binding.root)
        borderMarginHorizontal = resources.getDimensionPixelSize(
            if (mode == MODE_TAKE_SELFIE)
                R.dimen.selfie_border_margin_horizontal
            else R.dimen.id_card_border_margin_horizontal
        )
        borderMarginVertical = resources.getDimensionPixelSize(
            if (mode == MODE_TAKE_SELFIE)
                R.dimen.selfie_border_margin_vertical
            else R.dimen.id_card_border_margin_vertical
        )

        val params: ConstraintLayout.LayoutParams =
            binding.viewOverlay.layoutParams as ConstraintLayout.LayoutParams
        params.marginStart = if (mode == MODE_TAKE_SELFIE) 0 else borderMarginHorizontal
        params.marginEnd = if (mode == MODE_TAKE_SELFIE) 0 else borderMarginHorizontal
        params.topMargin = if (mode == MODE_TAKE_SELFIE) 0 else borderMarginVertical
        params.bottomMargin = if (mode == MODE_TAKE_SELFIE) 0 else borderMarginVertical
        binding.viewOverlay.layoutParams = params

        binding.viewOverlay.layoutParams = params
        binding.textViewMessage.text =
            if (mode == MODE_TAKE_SELFIE)
                ""
            else "Định vị chuẩn CMND/CCCD trong khung"
        binding.sStatusDetect.isChecked = enableAutoCapture
        binding.sStatusDetect.setOnCheckedChangeListener { _, isChecked ->
            Application.enableAuto = isChecked
            if (!isChecked) {
                analyzeTimer?.cancel()
            }
            enableAutoCapture = isChecked
            isCaptured = false
            startDetectionProcessor(!isChecked)
        }
    }

    private fun capture() {
        if (!isCaptured) {
            isCaptured = true
            analyzeTimer?.cancel()
            cameraManager.doAction()
        }
    }

    @SuppressLint("UnsafeOptInUsageError")
    private fun startCamera() {
        val cameraExecutor = ContextCompat.getMainExecutor(this)

        cameraManager = CameraManager.Builder()
            .setActivity(this)
            .setCameraPreview(binding.previewView)
            .setCameraMode(CameraMode.MODE_PHOTO)
            .setCameraExecutor(cameraExecutor)
            .setCameraSelectorOption(
                if (mode == MODE_TAKE_SELFIE)
                    CameraOption.LENS_FACING_FRONT else CameraOption.LENS_FACING_BACK
            )
            .setAspectRatio(
                if (mode == MODE_TAKE_SELFIE)
                    Constants.RATIO_3_4 else Constants.RATIO_4_3
            )
            .setCallback(object : CameraManager.CameraCallback {
                override fun onSuccess(filePath: String) {
//                    cornersDetectionProcessor?.process(BitmapFactory.decodeFile(filePath), true)
                }

                override fun onError(errorMessage: String?) {
                    setActivityResult(false)
                }
            })
            .build()

        try {
            cameraManager.startCamera()
            startDetectionProcessor(false)
            hasStartedCamera = true
        } catch (e: Exception) {
            Toast.makeText(this, "Không thể khởi động Camera!", Toast.LENGTH_SHORT).show()
            setResult(RESULT_CANCELED)
        }
    }


    private fun setActivityResult(
        success: Boolean,
        bitmap: Bitmap? = null,
        idCardType: String? = null
    ) {
        if (success && bitmap != null) {
            val filePath = this.writeBitmapToFile(bitmap, "IMG_")
            val intent = Intent()
            intent.putExtra(Constants.KEY_BITMAP, filePath)
            intent.putExtra(Constants.KEY_VERIFY_STEP, verifyProfileStep)
            intent.putExtra(Constants.KEY_ID_CARD_TYPE, idCardType)
            setResult(RESULT_OK, intent)
        } else {
            Toast.makeText(this, "Phân tích dữ liệu thất bại!", Toast.LENGTH_SHORT)
                .show()
            setResult(RESULT_CANCELED)
            analyzeTimerTask?.let { startTimer(it) }
        }
        finish()
    }

    private fun startDetectionProcessor(untEnableAuto: Boolean) {
        overlayTintList = ContextCompat.getColorStateList(
            this,
            R.color.green
        )!!

        if (mode == MODE_TAKE_SELFIE) {
            faceDetectionProcessor =
                FaceContourDetectionProcessor(
                    object : FaceContourDetectionProcessor.Callback {
                        override fun onAnalyzed(bitmap: Bitmap, analyzeResult: AnalyzeResult) {
                            onDetectedFaceSuccessfully(bitmap, analyzeResult == AnalyzeResult.OK)
                        }
                    },
                    borderMarginHorizontal.toFloat(),
                    borderMarginVertical.toFloat()
                )
        } else {
            cornersDetectionProcessor = CardCornersDetectionProcessor(
                this,
                object : CardCornersDetectionProcessor.Callback {
                    override fun onDetectedIdCardBitmap(result: AlignCardResult) {
                        onDetectedIdCardSuccessfully(result)
                    }

                    override fun onAnalyzedBitmap(result: AlignCardResult?, takeAPhoto: Boolean) {
                        if (takeAPhoto) {
                            isCaptured = false
                            if (result == null) {
                                Toast.makeText(
                                    this@CameraPreviewActivity,
                                    "Không thành công, xin thử lại.",
                                    Toast.LENGTH_SHORT
                                ).show()
                            } else {
                                setActivityResult(true, result.bitmap, result.cardType)
                            }
                        }
                    }

                },
                borderMarginHorizontal.toFloat(),
                borderMarginVertical.toFloat(),
                mode == MODE_CAPTURE_FRONT_ID_CARD,
                enableAutoCapture
            )
        }
        if (!untEnableAuto) {
            startAnalyzing()
        }
    }

    private fun onDetectedFaceSuccessfully(bitmap: Bitmap, result: Boolean) {
        binding.viewOverlay.backgroundTintList = if (result) null else null
        if (result && !AlignIdImageHelper.isBlurredImage(bitmap, lowThreshold = true)) {
            faceDetectionProcessor?.stop()
            setActivityResult(true, bitmap, null)
        }
    }

    private fun onDetectedIdCardSuccessfully(result: AlignCardResult) {
        cornersDetectionProcessor?.close()
        binding.viewOverlay.backgroundTintList = overlayTintList
        setActivityResult(true, result.bitmap, result.cardType)
    }

    private fun startAnalyzing() {
        analyzeTimer?.cancel()

        if (faceDetectionProcessor == null && cornersDetectionProcessor == null) {
            return
        }
        analyzeTimerTask = object : TimerTask() {
            override fun run() {
                Handler(Looper.getMainLooper()).postDelayed({
                    count += 1
                }, 2000)
                runOnUiThread {
                    if (count != 0) {
                        faceDetectionProcessor?.analyze(binding.previewView)
                        cornersDetectionProcessor?.process(binding.previewView.bitmap, false)
                    }
                }
            }
        }
        startTimer(analyzeTimerTask as TimerTask)
    }

    private fun startTimer(analyzeTimerTask: TimerTask) {
        analyzeTimer = Timer()
        analyzeTimer!!.scheduleAtFixedRate(analyzeTimerTask, ANALYZE_PERIOD, ANALYZE_PERIOD)
    }

    override fun onPause() {
        super.onPause()

        analyzeTimer?.cancel()
    }

    override fun onResume() {
        super.onResume()
        if (hasStartedCamera) {
            startAnalyzing()
        }
    }

    private fun allPermissionsGranted() = REQUIRED_PERMISSIONS.all {
        ContextCompat.checkSelfPermission(
            baseContext, it
        ) == PackageManager.PERMISSION_GRANTED
    }

    private fun checkPermissions() {
        if (allPermissionsGranted()) {
            startCamera()
        } else {
            ActivityCompat.requestPermissions(
                this, REQUIRED_PERMISSIONS, REQUEST_CODE_PERMISSIONS
            )
        }
    }

    override fun onRequestPermissionsResult(
        requestCode: Int, permissions: Array<String>, grantResults:
        IntArray
    ) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)

        if (requestCode == REQUEST_CODE_PERMISSIONS) {
            if (allPermissionsGranted()) {
                startCamera()
            } else {
                finish()
            }
        }
    }

    override fun onDestroy() {
        super.onDestroy()
        faceDetectionProcessor?.stop()
        cornersDetectionProcessor?.close()
    }

    private fun getIntent(intent: Intent?) {
        if (intent != null) {
            verifyProfileStep =
                intent.getSerializableExtra(Constants.KEY_VERIFY_STEP) as VerifyProfileStep
            mode = when (verifyProfileStep) {
                VerifyProfileStep.SELFIE -> MODE_TAKE_SELFIE
                VerifyProfileStep.FRONT_ID_CARD -> MODE_CAPTURE_FRONT_ID_CARD
                VerifyProfileStep.BACK_ID_CARD -> MODE_CAPTURE_BACK_ID_CARD
            }
        }
    }

    private fun String?.cardTypeToString(): String {
        if (this == IdCardType.BACK_CITIZEN_IDENTIFICATION || this == IdCardType.BACK_IDENTITY_CARD
            || this == IdCardType.BACK_CHIP_BASED_CITIZEN_IDENTIFICATION || this == IdCardType.BACK_OTHER_CARD
        ) {
            return "Mặt sau"
        }
        return "Mặt trước"
    }

    class Contract : ActivityResultContract<VerifyProfileStep, Intent?>() {
        private var step: VerifyProfileStep = VerifyProfileStep.SELFIE
        private var idCardType: String = IdCardType.FRONT_CHIP_BASED_CITIZEN_IDENTIFICATION
        override fun createIntent(context: Context, input: VerifyProfileStep): Intent {
            step = input
            val intent = Intent(context, CameraPreviewActivity::class.java)
            intent.putExtra(Constants.KEY_DATA, step)
                .putExtra(Constants.KEY_ID_CARD_TYPE, idCardType)
                .addFlags(Intent.FLAG_ACTIVITY_NO_ANIMATION)
            return intent
        }

        override fun parseResult(resultCode: Int, intent: Intent?): Intent? {
            return if (resultCode == RESULT_OK) {
                return Intent()
                    .putExtra(Constants.KEY_DATA, step)
                    .putExtra(Constants.KEY_BITMAP, intent?.getStringExtra(Constants.KEY_BITMAP))
                    .putExtra(
                        Constants.KEY_ID_CARD_TYPE,
                        intent?.getStringExtra(Constants.KEY_ID_CARD_TYPE)
                    )
            } else null
        }
    }
}