package com.Ekyc.ekyc2022

import android.app.Activity
import android.content.Intent
import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.widget.Toast
import androidx.annotation.NonNull
import com.Ekyc.ekyc2022.facedetection.AnalyzeResult
import com.Ekyc.ekyc2022.facedetection.FaceContourDetectionProcessor
import com.Ekyc.ekyc2022.global.Constants
import com.Ekyc.ekyc2022.objectdetection.AlignCardResult
import com.Ekyc.ekyc2022.objectdetection.CardCornersDetectionProcessor

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import java.io.ByteArrayOutputStream
import io.flutter.plugin.common.PluginRegistry

/** Ekyc2022Plugin */
class Ekyc2022Plugin: FlutterPlugin, MethodCallHandler, ActivityAware, EventChannel.StreamHandler,
PluginRegistry.ActivityResultListener {
  private var activity: Activity? = null
  private var faceDetectionProcessor: FaceContourDetectionProcessor? = null
  private lateinit var channel: MethodChannel

  companion object {
    private val RTMP_STREAM_RESULT = 66646
    internal var events: EventChannel.EventSink? = null
    var byteArray: ByteArray? = null
  }

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    channel = MethodChannel(flutterPluginBinding.binaryMessenger, Constants.KEY_CHANNEL)
    channel.setMethodCallHandler(this)
  }

  override fun onDetachedFromEngine(binding: FlutterPlugin.FlutterPluginBinding) {
    channel.setMethodCallHandler(null)
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    println("------${call.method}")
    when (call.method) {
      "key_stream" -> {
        val intent = Intent(activity, MainActivity::class.java)
        activity?.startActivityForResult(intent, RTMP_STREAM_RESULT)
      }

      "key_dectect" -> {
        detectCard(call)
      }
      "key_analyze" -> {
        analyze(call)
      }
      Constants.KEY_DETECT_FONT_CARD -> {
        startActivityPreView(VerifyProfileStep.FRONT_ID_CARD)
      }
      Constants.KEY_DETECT_BACK_CARD -> {
        startActivityPreView(VerifyProfileStep.BACK_ID_CARD)
      }
      Constants.KEY_DETECT_FACE -> {
        startActivityPreView(VerifyProfileStep.SELFIE)
      }
      else -> {
        result.notImplemented()
      }
    }
  }

  private fun startActivityPreView(
    verifyProfileStep: VerifyProfileStep,
  ) {
    val intent = Intent(activity, CameraPreviewActivity::class.java)
    intent.putExtra(Constants.KEY_VERIFY_STEP, verifyProfileStep)
    activity?.startActivityForResult(intent, RTMP_STREAM_RESULT)
  }

  private fun analyze(methodCall: MethodCall) {
    val byteArray: ByteArray? = methodCall.argument<ByteArray>("byteImage")
    var borderMarginHorizontal = methodCall.argument<Double>("borderMarginHorizontal")
    var borderMarginVertical = methodCall.argument<Double>("borderMarginVertical")

    if (borderMarginHorizontal == null) {
      borderMarginHorizontal = 0.0
    }
    if (borderMarginVertical == null) {
      borderMarginVertical = 0.0
    }

    faceDetectionProcessor =
      FaceContourDetectionProcessor(
        object : FaceContourDetectionProcessor.Callback {
          override fun onAnalyzed(bitmap: Bitmap, analyzeResult: AnalyzeResult) {
            if (analyzeResult == AnalyzeResult.OK) {
              channel.invokeMethod(
                Constants.KEY_BITMAP_AFTER_DETECT,
                convertBitmapToByte(bitmap)
              )
            } else {
              channel.invokeMethod(
                Constants.KEY_BITMAP_AFTER_DETECT,
                null
              )
            }
          }
        },
        borderMarginHorizontal.toFloat(),
        borderMarginVertical.toFloat()
      )
    if (byteArray != null) {
      val convertByteToBitmap = convertByteToBitmap(byteArray)
      if (convertByteToBitmap != null) {
        faceDetectionProcessor?.analyze(convertByteToBitmap)
      } else {
        channel.invokeMethod(
          Constants.KEY_BITMAP_AFTER_DETECT,
          null
        )
      }
    }
  }

  private fun detectCard(methodCall: MethodCall) {
    val path = methodCall.argument<String>("path")
    var borderMarginHorizontal = methodCall.argument<Double>("borderMarginHorizontal")
    var borderMarginVertical = methodCall.argument<Double>("borderMarginVertical")
    val step = methodCall.argument<Int>("step")

    val bitmap = BitmapFactory.decodeFile(path)
    if (borderMarginHorizontal == null) {
      borderMarginHorizontal = 0.0
    }
    if (borderMarginVertical == null) {
      borderMarginVertical = 0.0
    }
    activity?.let {
      CardCornersDetectionProcessor(
        it,
        object : CardCornersDetectionProcessor.Callback {
          override fun onAnalyzedBitmap(result: AlignCardResult?, takeAPhoto: Boolean) {
//                        if (bitmap != null) {
//                            channel.invokeMethod(
//                                Constants.KEY_BITMAP_AFTER_DETECT,
//                                convertBitmapToByte(bitmap)
//                            )
//                        } else {
//                            channel.invokeMethod(
//                                Constants.KEY_BITMAP_AFTER_DETECT,
//                                null
//                            )
//                        }
          }

          override fun onDetectedIdCardBitmap(idCardInfo: AlignCardResult) {
            channel.invokeMethod(
              Constants.KEY_BITMAP_AFTER_DETECT,
              convertBitmapToByte(idCardInfo.bitmap)
            )
          }
        },
        borderMarginHorizontal.toFloat(),
        borderMarginVertical.toFloat(),
        getMode(step)
      ).process(bitmap, false)
    }
  }


  private fun convertBitmapToByte(bitmap: Bitmap): ByteArray {
    val stream = ByteArrayOutputStream()
    bitmap.compress(Bitmap.CompressFormat.PNG, 100, stream)
    val byteArray: ByteArray = stream.toByteArray()
    bitmap.recycle()
    return byteArray
  }

  private fun convertByteToBitmap(byteArray: ByteArray): Bitmap? {
    if (byteArray.isEmpty()) {
      return null
    }
    return BitmapFactory.decodeByteArray(byteArray, 0, byteArray.size)
  }

  private fun getMode(step: Int?): Boolean {
    return !(step == 1 || step == 2)
  }

  override fun onAttachedToActivity(binding: ActivityPluginBinding) {
    activity = binding.activity
    binding.addActivityResultListener(this)
  }

  override fun onDetachedFromActivityForConfigChanges() {

  }

  override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {

  }

  override fun onDetachedFromActivity() {

  }

  override fun onListen(o: Any, eventSink: EventChannel.EventSink) {
    events = eventSink
  }

  override fun onCancel(o: Any) {
    events = null
  }

  override fun onActivityResult(requestCode: Int, resultCode: Int, result: Intent?): Boolean {
    if (requestCode == RTMP_STREAM_RESULT) {
      if (resultCode == Activity.RESULT_OK) {
        if (result != null) {
          if (result.getBooleanExtra(Constants.KEY_CANCEL, false)) {
            channel.invokeMethod(Constants.KEY_CANCEL, Constants.KEY_CANCEL)
            return true
          }
        }
        if (result == null || result.getStringExtra(Constants.KEY_BITMAP).isNullOrEmpty()) {
          Toast.makeText(activity, "Không phân tích được dữ liệu!", Toast.LENGTH_SHORT)
            .show()
          return false;
        }
        val imageFilePath = result.getStringExtra(Constants.KEY_BITMAP)
        val idCardType = result.getStringExtra(Constants.KEY_ID_CARD_TYPE)
        val verifyProfileStep = result.getStringExtra(Constants.KEY_VERIFY_STEP)
        val data: HashMap<String, String> = HashMap()
        data[Constants.KEY_ID_CARD_TYPE] = idCardType ?: ""
        data[Constants.KEY_VERIFY_STEP] = verifyProfileStep ?: ""
        data[Constants.KEY_PATH_AFTER_DETECT] = imageFilePath ?: ""
        channel.invokeMethod(Constants.KEY_DATA_AFTER_DETECT, data)
      }
      return true
    }
    return false
  }

}

