package com.Ekyc.ekyc2022.global

import android.graphics.Bitmap
import org.opencv.android.Utils
import org.opencv.core.Core.*
import org.opencv.core.CvType
import org.opencv.core.Mat
import org.opencv.core.Rect
import org.opencv.core.Scalar
import org.opencv.imgproc.Imgproc
import org.opencv.imgproc.Imgproc.COLOR_RGBA2GRAY

object ImageHelper {
    private const val BLOCK = 60
    private const val BLURRED_THRESHOLD = 0.0

    fun isBlurredImage(bitmap: Bitmap): Boolean {
        val frame = Mat()
        Utils.bitmapToMat(bitmap, frame)
        Imgproc.cvtColor(frame, frame, COLOR_RGBA2GRAY)

        val cx = frame.cols() / 2
        val cy = frame.rows() / 2
        val fImage = Mat()
        frame.convertTo(fImage, CvType.CV_32F)
        val fourierTransform = Mat()
        dft(fImage, fourierTransform, DFT_SCALE or DFT_COMPLEX_OUTPUT)

        val q0 = Mat(fourierTransform, Rect(0, 0, cx, cy))
        val q1 = Mat(fourierTransform, Rect(cx, 0, cx, cy))
        val q2 = Mat(fourierTransform, Rect(0, cy, cx, cy))
        val q3 = Mat(fourierTransform, Rect(cx, cy, cx, cy))

        val tmp = Mat()
        q0.copyTo(tmp)
        q3.copyTo(q0)
        tmp.copyTo(q3)

        q1.copyTo(tmp)
        q2.copyTo(q1)
        tmp.copyTo(q2)

        fourierTransform
            .submat(Rect(cx - BLOCK, cy - BLOCK, 2 * BLOCK, 2 * BLOCK))
            .setTo(Scalar.all(0.0))

        val orgFFT = Mat()
        fourierTransform.copyTo(orgFFT)
        val p0 = Mat(orgFFT, Rect(0, 0, cx, cy))
        val p1 = Mat(orgFFT, Rect(cx, 0, cx, cy))
        val p2 = Mat(orgFFT, Rect(0, cy, cx, cy))
        val p3 = Mat(orgFFT, Rect(cx, cy, cx, cy))

        p0.copyTo(tmp)
        p3.copyTo(p0)
        tmp.copyTo(p3)

        p1.copyTo(tmp)
        p2.copyTo(p1)
        tmp.copyTo(p2)

        val invFFT = Mat()
        val logFFT = Mat()
        val maxVal: Double

        dft(orgFFT, invFFT, DFT_INVERSE or DFT_REAL_OUTPUT)

        absdiff(invFFT, Scalar.all(0.0), invFFT)
        val minMaxLoc = minMaxLoc(invFFT)
        maxVal = minMaxLoc.maxVal

        if (maxVal <= 0.0) {
            return true
        }

        invFFT.convertTo(invFFT, CvType.CV_32F)
        log(invFFT, logFFT)
        multiply(logFFT, Scalar.all(20.0), logFFT)

        val result = mean(logFFT)
        return result.`val`[0] <= BLURRED_THRESHOLD
    }
}