package com.Ekyc.ekyc2022.global

import android.content.Context
import android.graphics.Bitmap
import android.util.Base64
import com.Ekyc.ekyc2022.objectdetection.IdCardType
import io.reactivex.Single
import java.io.ByteArrayOutputStream
import java.io.File
import java.io.FileOutputStream

object Utils {
    private var desCCCDChip: String = "Định vị chuẩn Thẻ căn cước găn chip trong khung"
    private var desPassport: String = "Định vị chuẩn Hộ chiếu trong khung"
    private var desOther: String = "Định vị chuẩn Giấy tờ khác trong khung"
    private var desCMNDCCCD: String = "Định vị chuẩn CMND/CCCD trong khung"

    fun Bitmap?.releaseBitmap() {
        if (this != null && !isRecycled) {
            recycle()
        }
    }

    fun Context.writeBitmapToFile(bitmap: Bitmap, fileNamePrefix: String): String? {
        try {
            val storageDir = filesDir
            val file = File.createTempFile(
                fileNamePrefix,  /* prefix */
                ".jpg",  /* suffix */
                storageDir /* directory */
            )
            val fileOutputStream = FileOutputStream(file)
            bitmap.compress(
                Bitmap.CompressFormat.JPEG,
                100,
                fileOutputStream
            )
            fileOutputStream.close()
            file.toString()
            return file.path
        } catch (e: Exception) {
            return null
        }
    }

    fun Bitmap.toBase64String(): Single<String> {
        return Single.create {
            val byteArrayOutputStream = ByteArrayOutputStream()
            compress(Bitmap.CompressFormat.JPEG, 100, byteArrayOutputStream)
            val byteArray = byteArrayOutputStream.toByteArray()
            it.onSuccess(Base64.encodeToString(byteArray, Base64.DEFAULT))
        }
    }

    fun getDesCardDetect(idCardType: String?): String {
        when (idCardType) {
            IdCardType.FRONT_CHIP_BASED_CITIZEN_IDENTIFICATION -> {
                return desCCCDChip
            }
            IdCardType.BACK_CHIP_BASED_CITIZEN_IDENTIFICATION -> {
                return desCCCDChip
            }
            IdCardType.FRONT_PASSPORT_CARD -> {
                return desPassport
            }
            IdCardType.FRONT_OTHER_CARD -> {
                return desOther
            }
            IdCardType.BACK_OTHER_CARD -> {
                return desOther
            }
            IdCardType.FRONT_IDENTITY_CARD -> {
                return desCMNDCCCD
            }
            IdCardType.BACK_IDENTITY_CARD -> {
                return desCMNDCCCD
            }
            IdCardType.FRONT_CITIZEN_IDENTIFICATION -> {
                return desCMNDCCCD
            }
            IdCardType.BACK_CITIZEN_IDENTIFICATION -> {
                return desCMNDCCCD
            }
        }
        return desCMNDCCCD
    }
}